﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.CompilerServices;
using Windows.ApplicationModel.Resources.Core;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Media.Imaging;
using System.Collections.Specialized;

// The data model defined by this file serves as a representative example of a strongly-typed
// model that supports notification when members are added, removed, or modified.  The property
// names chosen coincide with data bindings in the standard item templates.
//
// Applications may use this model as a starting point and build on it, or discard it entirely and
// replace it with something appropriate to their needs.

namespace NewsVariableTile.Data
{
    /// <summary>
    /// Base class for <see cref="SampleDataItem"/> and <see cref="SampleDataGroup"/> that
    /// defines properties common to both.
    /// </summary>
    [Windows.Foundation.Metadata.WebHostHidden]
    public abstract class SubGroupItemDataCommon : NewsVariableTile.Common.BindableBase
    {
        private static Uri _baseUri = new Uri("ms-appx:///");

        public SubGroupItemDataCommon(String uniqueId, String title, String subtitle, String imagePath, String description)
        {
            this._uniqueId = uniqueId;
            this._title = title;
            this._subtitle = subtitle;
            this._description = description;
            this._imagePath = imagePath;
        }

        private string _uniqueId = string.Empty;
        public string UniqueId
        {
            get { return this._uniqueId; }
            set { this.SetProperty(ref this._uniqueId, value); }
        }

        private string _title = string.Empty;
        public string Title
        {
            get { return this._title; }
            set { this.SetProperty(ref this._title, value); }
        }

        private string _subtitle = string.Empty;
        public string Subtitle
        {
            get { return this._subtitle; }
            set { this.SetProperty(ref this._subtitle, value); }
        }

        private string _Level = string.Empty;
        public string Level
        {
            get { return this._Level; }
            set { this.SetProperty(ref this._Level, value); }
        }

        private string _description = string.Empty;
        public string Description
        {
            get { return this._description; }
            set { this.SetProperty(ref this._description, value); }
        }

        private ImageSource _image = null;
        private String _imagePath = null;
        public ImageSource Image
        {
            get
            {
                if (this._image == null && this._imagePath != null)
                {
                    this._image = new BitmapImage(new Uri(SubGroupItemDataCommon._baseUri, this._imagePath));
                }
                return this._image;
            }

            set
            {
                this._imagePath = null;
                this.SetProperty(ref this._image, value);
            }
        }

        public void SetImage(String path)
        {
            this._image = null;
            this._imagePath = path;
            this.OnPropertyChanged("Image");
        }

        public override string ToString()
        {
            return this.Title;
        }
    }

    /// <summary>
    /// Generic item data model.
    /// </summary>
    public class SubGroupItemDataItem : SubGroupItemDataCommon
    {
        public SubGroupItemDataItem(String uniqueId, String title, String subtitle, String imagePath, String description, String content, SubGroupItemDataGroup group)
            : base(uniqueId, title, subtitle, imagePath, description)
        {
            this._content = content;
            this._group = group;
        }

        private string _content = string.Empty;
        public string Content
        {
            get { return this._content; }
            set { this.SetProperty(ref this._content, value); }
        }

        private SubGroupItemDataGroup _group;
        public SubGroupItemDataGroup Group
        {
            get { return this._group; }
            set { this.SetProperty(ref this._group, value); }
        }
    }

    /// <summary>
    /// Generic group data model.
    /// </summary>
    public class SubGroupItemDataGroup : SubGroupItemDataCommon
    {
        public SubGroupItemDataGroup(String uniqueId, String title, String subtitle, String imagePath, String description)
            : base(uniqueId, title, subtitle, imagePath, description)
        {
            Items.CollectionChanged += ItemsCollectionChanged;
        }

        private void ItemsCollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            // Provides a subset of the full items collection to bind to from a GroupedItemsPage
            // for two reasons: GridView will not virtualize large items collections, and it
            // improves the user experience when browsing through groups with large numbers of
            // items.
            //
            // A maximum of 12 items are displayed because it results in filled grid columns
            // whether there are 1, 2, 3, 4, or 6 rows displayed

            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    if (e.NewStartingIndex < 12)
                    {
                        TopItems.Insert(e.NewStartingIndex, Items[e.NewStartingIndex]);
                        if (TopItems.Count > 12)
                        {
                            TopItems.RemoveAt(12);
                        }
                    }
                    break;
                case NotifyCollectionChangedAction.Move:
                    if (e.OldStartingIndex < 12 && e.NewStartingIndex < 12)
                    {
                        TopItems.Move(e.OldStartingIndex, e.NewStartingIndex);
                    }
                    else if (e.OldStartingIndex < 12)
                    {
                        TopItems.RemoveAt(e.OldStartingIndex);
                        TopItems.Add(Items[11]);
                    }
                    else if (e.NewStartingIndex < 12)
                    {
                        TopItems.Insert(e.NewStartingIndex, Items[e.NewStartingIndex]);
                        TopItems.RemoveAt(12);
                    }
                    break;
                case NotifyCollectionChangedAction.Remove:
                    if (e.OldStartingIndex < 12)
                    {
                        TopItems.RemoveAt(e.OldStartingIndex);
                        if (Items.Count >= 12)
                        {
                            TopItems.Add(Items[11]);
                        }
                    }
                    break;
                case NotifyCollectionChangedAction.Replace:
                    if (e.OldStartingIndex < 12)
                    {
                        TopItems[e.OldStartingIndex] = Items[e.OldStartingIndex];
                    }
                    break;
                case NotifyCollectionChangedAction.Reset:
                    TopItems.Clear();
                    while (TopItems.Count < Items.Count && TopItems.Count < 12)
                    {
                        TopItems.Add(Items[TopItems.Count]);
                    }
                    break;
            }
        }

        private ObservableCollection<SubGroupItemDataItem> _items = new ObservableCollection<SubGroupItemDataItem>();
        public ObservableCollection<SubGroupItemDataItem> Items
        {
            get { return this._items; }
        }

        private ObservableCollection<SubGroupItemDataItem> _topItem = new ObservableCollection<SubGroupItemDataItem>();
        public ObservableCollection<SubGroupItemDataItem> TopItems
        {
            get { return this._topItem; }
        }
    }

    /// <summary>
    /// Creates a collection of groups and items with hard-coded content.
    /// 
    /// SubGroupItemDataSource initializes with placeholder data rather than live production
    /// data so that sample data is provided at both design-time and run-time.
    /// </summary>
    public sealed class SubGroupItemDataSource
    {
        private static SubGroupItemDataSource _SubGroupItemDataSource = new SubGroupItemDataSource();

        private ObservableCollection<SubGroupItemDataGroup> _allGroups = new ObservableCollection<SubGroupItemDataGroup>();
        public ObservableCollection<SubGroupItemDataGroup> AllGroups
        {
            get { return this._allGroups; }
        }

        public static IEnumerable<SubGroupItemDataGroup> GetGroups(string uniqueId)
        {
            if (!uniqueId.Equals("AllGroups")) throw new ArgumentException("Only 'AllGroups' is supported as a collection of groups");

            return _SubGroupItemDataSource.AllGroups;
        }

        public static SubGroupItemDataGroup GetGroup(string uniqueId)
        {
            // Simple linear search is acceptable for small data sets
            var matches = _SubGroupItemDataSource.AllGroups.Where((group) => group.UniqueId.Equals(uniqueId));
            if (matches.Count() == 1) return matches.First();
            return null;
        }

        public static SubGroupItemDataItem GetItem(string uniqueId)
        {
            // Simple linear search is acceptable for small data sets
            var matches = _SubGroupItemDataSource.AllGroups.SelectMany(group => group.Items).Where((item) => item.UniqueId.Equals(uniqueId));
            if (matches.Count() == 1) return matches.First();
            return null;
        }

        public SubGroupItemDataSource()
        {
            String ITEM_CONTENT = String.Format("Item Content: {0}\n\n{0}\n\n{0}\n\n{0}\n\n{0}\n\n{0}\n\n{0}",
                        "Curabitur class aliquam vestibulum nam curae maecenas sed integer cras phasellus suspendisse quisque donec dis praesent accumsan bibendum pellentesque condimentum adipiscing etiam consequat vivamus dictumst aliquam duis convallis scelerisque est parturient ullamcorper aliquet fusce suspendisse nunc hac eleifend amet blandit facilisi condimentum commodo scelerisque faucibus aenean ullamcorper ante mauris dignissim consectetuer nullam lorem vestibulum habitant conubia elementum pellentesque morbi facilisis arcu sollicitudin diam cubilia aptent vestibulum auctor eget dapibus pellentesque inceptos leo egestas interdum nulla consectetuer suspendisse adipiscing pellentesque proin lobortis sollicitudin augue elit mus congue fermentum parturient fringilla euismod feugiat");

            var group1 = new SubGroupItemDataGroup("Group-1",
                    "",
                    "Group Subtitle: 1",
                    "Assets/DarkGray.png",
                    "Group Description: Lorem ipsum dolor sit amet, consectetur adipiscing elit. Vivamus tempor scelerisque lorem in vehicula. Aliquam tincidunt, lacus ut sagittis tristique, turpis massa volutpat augue, eu rutrum ligula ante a ante");
            group1.Items.Add(new SubGroupItemDataItem("Group-1-Item-1",
                    "Lorum Ipsum",
					"Mauris quis interdum vehicula",
					"Assets/HomePage/HomePageImage1.png",
					"Item Description: Pellentesque porta, mauris quis interdum vehicula, urna sapien ultrices velit, nec venenatis dui odio in augue. Cras posuere, enim a cursus convallis, neque turpis malesuada erat, ut adipiscing neque tortor ac erat.",
					ITEM_CONTENT,
                    group1));
            group1.Items.Add(new SubGroupItemDataItem("Group-1-Item-2",
                    "Lorum Ipsum",
					"Mauris quis interdum vehicula",
					"Assets/HomePage/HomePageImage2.png",
					"Item Description: Pellentesque porta, mauris quis interdum vehicula, urna sapien ultrices velit, nec venenatis dui odio in augue. Cras posuere, enim a cursus convallis, neque turpis malesuada erat, ut adipiscing neque tortor ac erat.",
					ITEM_CONTENT,
                    group1));
            group1.Items.Add(new SubGroupItemDataItem("Group-1-Item-3",
                    "Lorum Ipsum",
					"Mauris quis interdum vehicula",
					"Assets/HomePage/HomePageImage3.png",
					"Item Description: Pellentesque porta, mauris quis interdum vehicula, urna sapien ultrices velit, nec venenatis dui odio in augue. Cras posuere, enim a cursus convallis, neque turpis malesuada erat, ut adipiscing neque tortor ac erat.",
					ITEM_CONTENT,
                    group1));
            group1.Items.Add(new SubGroupItemDataItem("Group-1-Item-4",
                   "Lorum Ipsum",
					"Mauris quis interdum vehicula",
					"Assets/HomePage/HomePageImage4.png",
					"Item Description: Pellentesque porta, mauris quis interdum vehicula, urna sapien ultrices velit, nec venenatis dui odio in augue. Cras posuere, enim a cursus convallis, neque turpis malesuada erat, ut adipiscing neque tortor ac erat.",
					ITEM_CONTENT,
                    group1));
            group1.Items.Add(new SubGroupItemDataItem("Group-1-Item-5",
                    "Lorum Ipsum",
					"Mauris quis interdum vehicula",
					"Assets/HomePage/HomePageImage5.png",
					"Item Description: Pellentesque porta, mauris quis interdum vehicula, urna sapien ultrices velit, nec venenatis dui odio in augue. Cras posuere, enim a cursus convallis, neque turpis malesuada erat, ut adipiscing neque tortor ac erat.",
					ITEM_CONTENT,
                    group1));
            group1.Items.Add(new SubGroupItemDataItem("Group-1-Item-6",
                   "Lorum Ipsum",
					"Mauris quis interdum vehicula",
					"Assets/HomePage/HomePageImage6.png",
					"Item Description: Pellentesque porta, mauris quis interdum vehicula, urna sapien ultrices velit, nec venenatis dui odio in augue. Cras posuere, enim a cursus convallis, neque turpis malesuada erat, ut adipiscing neque tortor ac erat.",
					ITEM_CONTENT,
                    group1));
			 group1.Items.Add(new SubGroupItemDataItem("Group-1-Item-7",
                   "Lorum Ipsum",
					"Mauris quis interdum vehicula",
					"Assets/HomePage/HomePageImage7.png",
					"Item Description: Pellentesque porta, mauris quis interdum vehicula, urna sapien ultrices velit, nec venenatis dui odio in augue. Cras posuere, enim a cursus convallis, neque turpis malesuada erat, ut adipiscing neque tortor ac erat.",
					ITEM_CONTENT,
                    group1));
			group1.Items.Add(new SubGroupItemDataItem("Group-1-Item-8",
                   "Lorum Ipsum",
					"Mauris quis interdum vehicula",
					"Assets/HomePage/HomePageImage8.png",
					"Item Description: Pellentesque porta, mauris quis interdum vehicula, urna sapien ultrices velit, nec venenatis dui odio in augue. Cras posuere, enim a cursus convallis, neque turpis malesuada erat, ut adipiscing neque tortor ac erat.",
					ITEM_CONTENT,
                    group1));
			group1.Items.Add(new SubGroupItemDataItem("Group-1-Item-9",
                   "Lorum Ipsum",
					"Mauris quis interdum vehicula",
					"Assets/HomePage/HomePageImage9.png",
					"Item Description: Pellentesque porta, mauris quis interdum vehicula, urna sapien ultrices velit, nec venenatis dui odio in augue. Cras posuere, enim a cursus convallis, neque turpis malesuada erat, ut adipiscing neque tortor ac erat.",
					ITEM_CONTENT,
                    group1));
			 group1.Items.Add(new SubGroupItemDataItem("Group-1-Item-10",
                   "Lorum Ipsum",
					"Mauris quis interdum vehicula",
					"Assets/HomePage/HomePageImage10.png",
					"Item Description: Pellentesque porta, mauris quis interdum vehicula, urna sapien ultrices velit, nec venenatis dui odio in augue. Cras posuere, enim a cursus convallis, neque turpis malesuada erat, ut adipiscing neque tortor ac erat.",
					ITEM_CONTENT,
                    group1));
			group1.Items.Add(new SubGroupItemDataItem("Group-1-Item-11",
                   "Lorum Ipsum",
					"Mauris quis interdum vehicula",
					"Assets/HomePage/HomePageImage11.png",
					"Item Description: Pellentesque porta, mauris quis interdum vehicula, urna sapien ultrices velit, nec venenatis dui odio in augue. Cras posuere, enim a cursus convallis, neque turpis malesuada erat, ut adipiscing neque tortor ac erat.",
					ITEM_CONTENT,
                    group1));
			group1.Items.Add(new SubGroupItemDataItem("Group-1-Item-12",
                   "Lorum Ipsum",
					"Mauris quis interdum vehicula",
					"Assets/HomePage/HomePageImage12.png",
					"Item Description: Pellentesque porta, mauris quis interdum vehicula, urna sapien ultrices velit, nec venenatis dui odio in augue. Cras posuere, enim a cursus convallis, neque turpis malesuada erat, ut adipiscing neque tortor ac erat.",
					ITEM_CONTENT,
                    group1));
         
         
            this.AllGroups.Add(group1);

           
          
        }
    }
}